#include <mylib_priv.h>
#include <mylib_regression.h>

#include <stdio.h>
#include <string.h>

struct module_set functions[] = {
	{ "log2", regress_log2 },
	{ "clog2", cregress_log2 },
	{ "ceillog2", regress_ceillog2 },
	{ "cceillog2", cregress_ceillog2 },
	{ "firstclearbit", regress_firstclearbit },
};

void
dumpmodules(void)
{
	int p, n;
	int i;
	int rows = 80;	/* XXX - later add code to set this properly */

	printf("Modules:\n\t");
	rows -= 8;
	p = 0;
	n = 1;
	for (i = 0; i * sizeof *functions < sizeof(functions); i++) {
		if (n != 1)
			printf(",");
		else
			n = 0;
		if (p + strlen(functions[i].name) + 2 >= rows) {
			printf("\n\t");
			p = 0;
		}
		printf("%s%s", p == 0 ? "" : " ",
		    functions[i].name);
		p += strlen(functions[i].name) +
		    ((p == 0) ? 1 : 2);
	}
	printf("\n");
}

struct module_set *
findmodule(char *str)
{
	int i;

	for (i = 0; i * sizeof *functions < sizeof(functions);
	    i++)
		if (strcmp(functions[i].name, str) == 0)
			break;
	if (i * sizeof *functions >= sizeof(functions))
		return NULL;

	return &functions[i];
}

void
doregress(struct module_set a)
{
	int i;

	if ((i = a.test()) != 0)
		printf("regression test of %s return %d.\n", a.name, i);
	else
		printf("regression test of %s successful.\n", a.name);
}

void
main(int argc, char *argv[])
{
	int i;
	struct module_set *ms;

	argc--; argv++;

	while (argc) {
		if (strcmp(*argv, "list") == 0)
			dumpmodules();
		else if (strcmp(*argv, "all") == 0)
			for (i = 0; i * sizeof *functions < sizeof(functions);
			    i++)
				doregress(functions[i]);
		else
			if ((ms = findmodule(*argv)) == NULL)
				printf("%s not a valid test.\n", *argv);
			else
				doregress(*ms);
		argc--; argv++;
	}
}
